#!/bin/bash 

#print help if no parameter is entered on the command line
if [ $# -lt 1 ]; then echo "Usage: $0 load | quit |record | pause | status"; exit 0 ;fi

#to get here there is at least 1 parameter on the command line 
#we can test to see if raspivid and motion are loaded by finding the pid#

	pidnum=$( pgrep raspivid )				#find pid for raspivid if it exists
	pidnum1=$( pgrep motion )				#find pid for motion if it exists

#perform actions based on command line parameter $1
case $1 in
load)		#if raspivid is already running, warn user, if not then start the process
	if [ $pidnum ] || [ $pidnum1 ]; then	# || is OR operator, exit 1 if either process running
		if [ $pidnum ]; then  echo "raspivid process already running" ; fi
		if [ $pidnum1 ]; then  echo "motion process already running" ; fi
		exit 1 
	else					# both process not running
		echo "Starting raspivid, this will overwrite video files if they exist"
		echo "paused" > /home/pi/camera/picam_state.txt
		# start raspivid as root, even if we close this terminal window it will continue to run
#		raspivid -w 800 -h 600 -i pause -p 0,0,320,240 -o /media/data/video/test%02d.h264 -sp -s -t 0 &
		raspivid -w 800 -h 600 -i pause -n -o /media/data/video/test%02d.h264 -sg 10000 -s -t 0 &
		echo "Starting motion" 
		motion  -n  -c /home/pi/camera/motion.conf.test 2> /dev/null  &		#dump text to null
		# setup the webcam to fixed focus
		uvcdynctrl -d /dev/video0 -s "Focus, Auto" 0 & 
		exit 0 
	fi
;;
quit)		#Kill raspivid/motion process if there is an instance already running, we don´t care who started it
	if [ $pidnum ] || [ $pidnum1 ]; then 	# || is the OR operator
		echo "Killing processes"
		#Kill raspivid
		sudo killall raspivid
		echo "stopped" > /home/pi/camera/picam_state.txt
		#kill motion process
		sudo killall motion 
		exit 0
	else 
		echo "No processes to kill"
		echo "stopped" > /home/pi/camera/picam_state.txt
		exit 1  
	fi
;;
record)		#If raspivid is running and paused, start recording
	if [ -z $pidnum ]; then 
		echo "raspivid process not running"
		echo "stopped" > /home/pi/camera/picam_state.txt
		exit 1
	fi
	read state < /home/pi/camera/picam_state.txt
	if [ $state = "paused" ]; then
		echo "Starting recording"
		sudo kill -usr1 $pidnum
		echo "recording" > /home/pi/camera/picam_state.txt
		echo "recording" >> /home/pi/camera/securitylog.txt
		exit 0
	fi
	if [ $state = "recording" ]; then
		echo "Already recording"
	else
		echo "State errror"
	fi
		exit 1
;;
pause)		#if raspivid is recording, pause it
	if [ -z $pidnum ]; then 
		echo "raspivid process not running"
		echo "stopped" > /home/pi/camera/picam_state.txt  
		exit 1
	fi
	read state < picam_state.txt
	if [ $state = "recording" ]; then
		echo "Pausing recording"
		sudo kill -usr1 $pidnum
		echo "paused" > /home/pi/camera/picam_state.txt
		echo "pausing" >> /home/pi/camera/securitylog.txt
		exit 0
	fi
	if [ $state = "paused" ]; then
		echo "Already paused"
	else
		echo "State errror"
	fi
		exit 1
;;
status)		#echo current raspivid state, recording, paused or stopped and motion state
	if [ -z $pidnum ] || [ -z $pidnum1 ]; then 			# if a or b are null ("")
		if [ -z $pidnum ]; then
			echo "raspivid process not running"
			echo "stopped" > /home/pi/camera/picam_state.txt
		else
			echo "raspivid process running"
		fi  		
		if [ -z $pidnum1 ]; then
			echo "motion process not running"
		else
			echo "motion process running"
		fi  
		exit 1
	fi
	if [ $pidnum ]; then echo "raspivid process running" ; fi
	if [ $pidnum1 ]; then echo "motion process running" ; fi
	read state < picam_state.txt
	if [ $state = "stopped" ]; then
		echo "State errror"
		exit 1
	else
		echo "raspivid is " $state
		exit 0
	fi
;;
*)
	echo "Unknown command" $1
	echo "Usage: $0 load | quit |record | pause | status"
	exit 1
;;
esac

exit 0
