#!/usr/bin/python3

import telnetlib
from time import sleep

#global variables
HOST = "localhost"
PORT = "4212"
strPlaylist="/home/pi/radio/playlist"
stationarray=[]
stationcount=0
SessionFlag=False
tn=telnetlib.Telnet()  #session reference used by multiple functions
inbuf=[]
outbuf=[]
userloop=True
debug=False

def ParsePlaylist(filename):
    stationlist=[]
    printlist=[]
    global stationcount
    try:
        with open(filename, "r") as input:
            for line in input:
                if ("," in line ):
                    line=line.replace("  ","")
                    line= line.split(",")
                    stationlist=stationlist + line
                    printlist=line
                    stationcount += 1
                    print(stationcount, printlist[0])
            input.close()
    except IOError:
        print ("Unable to open ", filename)
        return(False)   
    return (stationlist)


def TelnetTx( outstring ):
    """Telnet send command, return True=no Error"""    
    global tn
    global outbuf
    global inbuf
    global SessionFlag
    try:
        outbuf=outstring
        tn.write( outbuf.encode("ascii") )
        if debug : print("Write command = ", outstring)
        return (True)
    except:
        print("There was an exception with a Telnet Wr command")
        return(False)

def TelnetRx( outstring ):
    """Telnet read_until command, return inbuf=no Error"""    
    global tn
    global outbuf
    global inbuf
    global SessionFlag
    try:
        outbuf=outstring
        inbuf=tn.read_until( outbuf.encode("ascii"), 1)
        if debug : print("Read until command = ", outstring)
        if debug : print("Read until return string = ", inbuf)
        return (True)
    except:
        print("There was an exception with a Telnet Rd command")
        return(False)

def CreateSession( host, port, timeout ):
    try:
        global tn
        tn.open(host, port, timeout)
        TelnetRx("Password:")
        TelnetTx("admin\n")
        TelnetRx("> ")
        return(True)
    except:
        print("There was a problem connecting to the VLC server", host, port)
        return(False)


def ClearPlaylist():
    loopcount=5
    global outbuf
    global inbuf
    global SessionFlag
    if (SessionFlag == True):
        global tn
        TelnetTx("clear\n")
        TelnetRx("> ")
        TelnetTx("is_playing\n")
        TelnetRx("> ")          #inbuf contains is_playing response
        while inbuf[0] == 49 and loopcount:     #49= ascii 1
            loopcount -= 1
            if debug : print("The is_playing flag is 1")
            if debug : print("Waiting for is_playing to drop")
            TelnetTx("is_playing\n")
            TelnetRx("> ")                      #test again
        if debug : print("The is_playing flag is 0")
        return(True)
    else:
        print("Session is closed")
        return(False)


def AddStationtoPlaylist( stationnum, retryflag ):
    """Clear playist and add new URL entry, if retryflag is True then it's a retry"""
    loopcount=15        #loop for maximum 15 * 0.15 seconds = 2.25 seconds max
    global outbuf
    global inbuf
    global SessionFlag
    global tn
    if (SessionFlag == True):
        if retryflag: print("Retrying....")
        ClearPlaylist()
        if debug : print("Trying to Add station", stationnum)
        TelnetTx("add " + stationarray[ ( (stationnum * 2) - 1) ] + "\n")
        TelnetRx("> ")
        TelnetTx("is_playing\n")
        TelnetRx("> ")
        #we need to check that the vlc is_playing flag drops to zero
        while inbuf[0] == 48 and loopcount: 
            loopcount -= 1
            if debug : print("Loopcount = ", loopcount)
            if debug : print("The is_playing flag is still 0")
            if debug : print("Waiting for is_playing to set")
            TelnetTx("is_playing\n")
            if debug : print("Test is_playing flag")
            TelnetRx("> ")      #loop to test flag again without any delay
            #when is_playing is set to one we know that vlc is responding
            #to the new station URL
        if debug : print("The is_playing flag is now 1")
        TelnetTx("is_playing\n")
        TelnetRx("> ")
        while inbuf[0] == 49 and loopcount:         #49 = ascii 1
            loopcount -= 1
            if debug : print("Wait to see if is_playing is stable, Loopcount = ", loopcount)
            sleep (0.15)
            TelnetTx("is_playing\n")
            TelnetRx("> ")
            if inbuf[0] == 48: AddStationtoPlaylist( stationnum, True )
            if inbuf[0] == 49: continue
        return(True)
    else:
        print("Session is closed")
        return(False)
        
        
def PlayRadio():
    global stationarray
    global stationcount
    global SessionFlag
    global strPlaylist
    global userloop
    #Parse the playlist file
    tmp = ParsePlaylist(strPlaylist)
    if tmp == False:     #return is either the playlist or a fail flag
        print("Playlist error .....exiting")
        exit()
    else:
        stationarray=tmp
    if (stationcount != 0):
        SessionFlag = CreateSession(HOST, PORT, 2)
        if not SessionFlag:
            print("Unable to connect to VLC...exiting")
            exit()
        #If we get here we have a playlist and can talk to VLC
        AddStationtoPlaylist( 1, False )       
        print("Station 1 is the default")
    else:
        print("No stations to play ....exiting")
        exit()
    #Once the radio is running we just sit in a tight loop for user input
    while userloop:     #If you set userloop=False you can debug in IDLE
        try:
            x=int(input("Enter a new station number or 0 to exit: "))
        except ValueError:
                  print("Enter a number.. ")
        else:
                  if x == 0:
                      break
                  if 1 <= x <= (stationcount):
                      AddStationtoPlaylist(x, False)
                  else:
                      print("Out of range, try again: ")
    exit()
                            
       
#Start the radio

PlayRadio()


