#!/usr/bin/python3

from tkinter import *
import time, telnetlib

#global Python variables
HOST = "localhost"
PORT = "4212"
strPlaylist="/home/pi/radio/playlist"
stationarray=[]
stationtuple=()
stationcount=0
stationnum=1
SessionFlag=False
tn=telnetlib.Telnet()       #session reference used by multiple functions
userloop=True
debug=False


#Define the TKinter window
root=Tk() #Creates the window reference
root.wm_title("Internet Radio") # Window title
root.config(background = "#FFFFF0")

#TKinter variables, these can be bound to a control
vtime=StringVar(value="Time")
vstatus=StringVar(value="...........")
vspinnum=StringVar(value="")


#Function definitions

def ParsePlaylist(filename):
    stationlist=[]
    printlist=[]
    global stationcount
    global stationtuple
    try:
        with open(filename, "r") as input:
            temptext=""
            for line in input:
                if ("," in line ):
                    line=line.replace("  ","")
                    line= line.split(",")
                    stationlist=stationlist + line
                    printlist=line
                    stationcount += 1
                    temptext=temptext + ( '"' + str(stationcount) + ") " + printlist[0] + '" ' )
                    stationtuple=temptext
                    if debug : print(stationcount, printlist[0])
            input.close()
    except IOError:
        print ("Unable to open ", filename)
        exit()   
    return (stationlist)


def TelnetTx( outstring ):
    """Telnet send command, return True=no Error"""    
    global tn
    global outbuf
    global inbuf
    global SessionFlag
    try:
        outbuf=outstring
        tn.write( outbuf.encode("ascii") )
        if debug : print("Write command = ", outstring)
        return (True)
    except:
        print("There was an exception with a Telnet Wr command")
        return(False)

def TelnetRx( outstring ):
    """Telnet read_until command, return inbuf=no Error"""    
    global tn
    global outbuf
    global inbuf
    global SessionFlag
    try:
        outbuf=outstring
        inbuf=tn.read_until( outbuf.encode("ascii"), 1)
        if debug : print("Read until command = ", outstring)
        if debug : print("Read until return string = ", inbuf)
        return (True)
    except:
        print("There was an exception with a Telnet Rd command")
        return(False)

def CreateSession( host, port, timeout ):
    try:
        global tn
        tn.open(host, port, timeout)
        TelnetRx("Password:")
        TelnetTx("admin\n")
        TelnetRx("> ")
        return(True)
    except:
        print("There was a problem connecting to the VLC server", host, port)
        return(False)


def ClearPlaylist():
    loopcount=5
    global outbuf
    global inbuf
    global SessionFlag
    if (SessionFlag == True):
        global tn
        TelnetTx("clear\n")
        TelnetRx("> ")
        TelnetTx("is_playing\n")
        TelnetRx("> ")          #inbuf contains is_playing response
        while inbuf[0] == 49 and loopcount:     #49= ascii 1
            loopcount -= 1
            if debug : print("The is_playing flag is 1")
            if debug : print("Waiting for is_playing to drop")
            TelnetTx("is_playing\n")
            TelnetRx("> ")                      #test again
        if debug : print("The is_playing flag is 0")
        return(True)
    else:
        print("Session is closed")
        return(False)


def AddStationtoPlaylist( stationnum, retryflag ):
    """Clear playist and add new URL entry, if retryflag is True then it's a retry"""
    loopcount=15        #loop for maximum 15 * 0.15 seconds = 2.25 seconds max
    global outbuf
    global inbuf
    global SessionFlag
    global tn
    global vstatus
    if (SessionFlag == True):
        if retryflag:
            print("Retrying....")
            #set the UI status field and force an update
            vstatus.set("Retrying....")
            root.update() 
        ClearPlaylist()
        if debug : print("Trying to Add station", stationnum)
        TelnetTx("add " + stationarray[ ( (stationnum * 2) - 1) ] + "\n")
        TelnetRx("> ")
        TelnetTx("is_playing\n")
        TelnetRx("> ")
        #we need to check that the vlc is_playing flag drops to zero
        while inbuf[0] == 48 and loopcount: 
            loopcount -= 1
            if debug : print("Loopcount = ", loopcount)
            if debug : print("The is_playing flag is still 0")
            if debug : print("Waiting for is_playing to set")
            TelnetTx("is_playing\n")
            if debug : print("Test is_playing flag")
            TelnetRx("> ")      #loop to test flag again without any delay
            #when is_playing is set to one we know that vlc is responding
            #to the new station URL
        if debug : print("The is_playing flag is now 1")
        TelnetTx("is_playing\n")
        TelnetRx("> ")
        while inbuf[0] == 49 and loopcount:         #49 = ascii 1
            loopcount -= 1
            if debug : print("Wait to see if is_playing is stable, Loopcount = ", loopcount)
            time.sleep (0.15)
            TelnetTx("is_playing\n")
            TelnetRx("> ")
            #If is_playing=48=zero then we will do a retry
            if inbuf[0] == 48: AddStationtoPlaylist( stationnum, True )
            #if is_playing=49=one we assume all is ok
            if inbuf[0] == 49:
                vstatus.set("...........")
                continue
        return(True)
    else:
        print("Session is closed")
        return(False)

def stopRadio():
    global SessionFlag
    if (SessionFlag == True):
        try:
            global tn
            TelnetTx("stop\n")
            print("Trying to stop")
            TelnetRx("> ")
            return(True)
        except:
            print("There was a problem sending stop command")
            return(False)
    else:
        print("Session is closed")

        
def PlayRadio():
    global stationarray
    global stationcount
    global SessionFlag
    global strPlaylist
    global userloop
    if (stationcount != 0):
        SessionFlag = CreateSession(HOST, PORT, 2)
        if not SessionFlag:
            print("Unable to connect to VLC...exiting")
            exit()
        #If we get here we have a playlist and can talk to VLC
        AddStationtoPlaylist( 1, False )       
        print("Station 1 is the default")
        return(True)
    else:
        print("No stations to play ....exiting")
        exit()




#Event handlers for interface elements
def playbtnPress():
    if debug : print ("Play Button Pressed")
    global stationnum
    AddStationtoPlaylist(stationnum, False)

def stopbtnPress():
    if  debug : print ("Stop Button Pressed")
    TelnetTx("pause\n")
    
def spinbtnPress():
    global stationnum
    line=vspinnum.get()
    line= line.split(")")
    if debug : print("Spin cursor on ", stationnum, line)
    stationnum=int(line[0])
    
def update_clock():
    vtime.set( time.strftime(" %a %B %d  %l:%M:%S %p "))
    root.after(500, update_clock)

def buildwindow():

    #primaryFrame in root window
    primaryFrame = Frame(root)
    primaryFrame.grid(row=0, column=0, sticky=W+N, padx=10, pady=10 )
    primaryFrame.grid_columnconfigure(0)

    #Labels in primaryFrame
    Label(primaryFrame, text="Station").grid(row=0, column=0, padx=10, pady=10)
    statusLabel= Label(primaryFrame, textvariable=vstatus).grid(row=12, column=0, padx=10, pady=10)

    #Contents of primaryFrame

    playBtn = Button(primaryFrame, text="Play", command=playbtnPress, width=10)
    playBtn.grid(row=11, column=0, sticky=W, padx=10, pady=2)

    stopBtn = Button(primaryFrame, text="Stop", command=stopbtnPress, width=10)
    stopBtn.grid(row=11, column=0, sticky=E, padx=10, pady=2)


    timeLabel= Label(primaryFrame, textvariable=vtime, font=("Helvetica", 22), fg="white", bg="black" ).grid(row=13, column=0, padx=10, pady=10)

    #Create a sub frame within primaryFrame to hold the listbox and scrollbar
    subFrame = Frame(primaryFrame )
    subFrame.grid(row=1, column=0, rowspan=10, padx=10, pady=2)


    stationSpin=Spinbox(subFrame, width=25, state="readonly", textvar=vspinnum, command=spinbtnPress, values=stationtuple )
    stationSpin.config(font=("Helvetica", 22), wrap=True, repeatdelay=1000, repeatinterval=250)
    stationSpin.pack()
    #
    #End of primaryFrame contents


#Run  code to read Playlist file and connect to VLC before building GUI
#Errors cause exit() before the GUI appears

#Do this first before anything else to get the spinbox 
#and stationarray values
stationarray = ParsePlaylist(strPlaylist)

PlayRadio()

#Now start the GUI elements

buildwindow()

update_clock()          #Kick off the clock

root.mainloop()         #start window event loop


