#!/usr/bin/python3

from tkinter import *
from time import strftime
import RPi.GPIO as GPIO


GPIO.setmode(GPIO.BCM)
GPIO.setup(26, GPIO.OUT, pull_up_down=GPIO.PUD_UP, initial=1)
GPIO.setup(21, GPIO.IN, pull_up_down=GPIO.PUD_UP)

#Python variable
debug=True
if ( debug ): print ("Library Revision", GPIO.VERSION)


#Define the TKinter window
root = Tk() #Makes the window
root.wm_title("GPIO Test") # Window title
root.config(background = "#FFFFF0")

#TKinter variables, these can be bound to a control
vtime=StringVar(value="Time")
Rbvalue=IntVar(value=0)
Pvalue=StringVar(value=".......")

#Event handlers for interface elements
def edgebtnPress():
    if ( debug ): print ("Edge Button Pressed")
    Pvalue.set(value="Press the Button...")
    edgewait()

def edgewait():
    if ( debug ): print ("Edge wait start..")
    GPIO.setup(21, GPIO.IN, pull_up_down=GPIO.PUD_UP)
    GPIO.wait_for_edge(21, GPIO.FALLING, bouncetime=50)
    if ( debug ): print ("Edge detected..")
    GPIO.cleanup(21)
    edgeidBtn.config(background="#00FF00")
    monoLED(500)
    
def eventbtnPress():
    if ( debug ): print ("Event Button Pressed")
    Pvalue.set(value="Press the Button...")
    GPIO.setup(21, GPIO.IN, pull_up_down=GPIO.PUD_UP)
    GPIO.add_event_detect(21, GPIO.FALLING, bouncetime=50)
    if ( debug ): print ("Waiting for event..")    
    evwait()

def evwait():
    if GPIO.event_detected(21):
        if ( debug ): print ("Event detected...")    
        GPIO.cleanup(21)
        eventidBtn.config(background="#00FF00")
        monoLED(500)
    else:
        root.after(20, evwait )      

def monoLED(timer):
    if ( debug ): print("LED turned on")
    GPIO.output(26, 0)
    Rbvalue.set(0)
    root.after(timer, monotimeout)      

def monotimeout():
    GPIO.output(26, 1)  
    edgeidBtn.config(background="#000000")
    eventidBtn.config(background="#000000")
    Pvalue.set(value=".......")


def update_clock():
    vtime.set( strftime(" %S "))
    root.after(100, update_clock)  

def exitbtnPress():
    if ( debug == 1 ): print ("Exit Button Pressed")
    GPIO.cleanup()
    root.destroy()
    exit()

    
#primaryFrame in root window
primaryFrame = Frame(root)
primaryFrame.grid(row=0, column=0, sticky=W+N, padx=10, pady=10 )
primaryFrame.grid_columnconfigure(0)

#Labels in primaryFrame
#Label(primaryFrame, text="LED Test").grid(row=0, column=0, padx=10, pady=10)

#Contents of primaryFrame

edgeBtn = Radiobutton(primaryFrame, text="Edge", variable=Rbvalue, value=1, command=edgebtnPress, indicatoron=0, width=10)
edgeBtn.grid(row=0, column=0, padx=10, pady=2)

eventBtn = Radiobutton(primaryFrame, text="Event", variable=Rbvalue, value=2, command=eventbtnPress, indicatoron=0, width=10)
eventBtn.grid(row=1, column=0, padx=10, pady=2)

edgeidBtn = Button(primaryFrame, text="Edge Indicator", state=DISABLED, width=10)
edgeidBtn.grid(row=0, column=1, padx=10, pady=2)
edgeidBtn.config(background="#000000")

eventidBtn = Button(primaryFrame, text="Event Indicator", state=DISABLED, width=10)
eventidBtn.grid(row=1, column=1, padx=10, pady=2)
eventidBtn.config(background="#000000")

timeLabel= Label(primaryFrame, textvariable=vtime, font=("Helvetica", 22), fg="white", bg="black" ).grid(row=2, column=0, padx=10, pady=10)
pressLabel= Label(primaryFrame, textvariable=Pvalue, width=14 ).grid(row=2, column=1, padx=10, pady=10)

exitBtn = Button(primaryFrame, text="Exit", command=exitbtnPress, width=10)
exitBtn.grid(row=2, column=3, padx=10, pady=2)

#End of primaryFrame contents


#Now start the GUI elements

update_clock()          #Kick off the clock

root.mainloop()         #start window event loop


