#!/usr/bin/python

from time import sleep
from random import randrange
from smbus import SMBus

# ==========================================================
# With help from Adafruit PCA9685 16-Channel PWM Servo Driver 
# ==========================================================

# RPi - PCA9685 Constants 
__i2cbus             = 0x01     #older Pi may use bus 0 = 0x00
__Gencall            = 0x00
__PCA9685add         = 0x40
__PCA9685rst         = 0x06     #GenCall(0x06) resets all PCA9685
__MODE1              = 0x00
__PRESCALE           = 0xFE
__LED0_ON_L          = 0x06     #first register in quad
__Neutral            = [ 0x00, 0x00, 0x72, 0x01 ]  #1.5mS=370=0x172
__Quadlen            = 0x04

#create i2c SMBus object reference
i2c = SMBus(__i2cbus)

#****************************************************
#SMBus functions

def busWritebyte( address, reg):
    #Adresses a single special register  
    try:
        return i2c.write_byte(address, reg)
    except IOError, err:
        if debug : print "Error on byte write ", address, reg, err
        return True   

def busReadbytedata (address, reg ):
    #Reads a single bye from a register
    try:
        return i2c.read_byte_data(address, reg)
    except IOError, err :
        if debug : print "IO Error busreadbytedata ", address, reg, err
        return True

def busWritebytedata (address, reg, data):
    #Writes a single byte to a register
    try:
        i2c.write_byte_data(address, reg, data)
        return False
    except IOError, err :
        if debug : print "IO Error buswritebyte ", address, reg, data, err
        return True

def busWritelist(address, reg, alist):      
    #Writes an array of bytes in a list (needs AI)
    try:
        i2c.write_i2c_block_data(address, reg, alist)
        return False
    except IOError, err :
        if debug : print "IO Error buswritelist ", address, reg, alist, err
        return True

def busReadlist(address, reg, listlen):
    #Read multiple bytes into an array (needs AI)
    try:
        return i2c.read_i2c_block_data(address, reg, listlen)
    except IOError, err :
        if debug : print "IO Error buswritelist ", address, reg, listlen, err
        return True

#******************************************
#PCA9685 specific functions
    
def PCA9685_reset():
    try:
        busWritebyte(__Gencall, __PCA9685rst) 
        return False
    except:
        print "Exception error"
        return True

def setPWMFreq(freq):
    # Sets the PWM frequency for this board
    prescale = 25000000.0    #25MHz rc osc
    prescale /= 4096.0       #12-bit resolution
    prescale /= float(freq)
    prescale -= 1.0
    prescale = int(prescale + 0.5)
    # the PCA9685 must be asleep to set the prescale divider count
    try:
        mode1data = busReadbytedata(__PCA9685add, __MODE1)   #read current MODE1 reg
        if not (mode1data & 0x10) :
            mode1data = mode1data | 0x10                         # sleep bit not set
            busWritebytedata(__PCA9685add, __MODE1, mode1data)   #MODE1 write sleep
        busWritebytedata(__PCA9685add, __PRESCALE, prescale)  #write prescale count
        mode1data=mode1data & 0xEF | 0x20          #MODE1 set AI, clear SLEEP
        busWritebytedata(__PCA9685add, __MODE1, mode1data)      #MODE1 write
        sleep(0.001)                                            #wait time for osc
        return False
    except:
        print "Exception error"
        return True

def setPWMquad(address, channel, alist):
    # Sets a single PWM channel, count can be 0-4095
    # Channels are 0-15 for individual settings and 61 for ALL_LED_ON_L
    # Quad register List is [ON_lsb, ON_msb, OFF_lsb, OFF_msb]
    try:
        busWritelist(address, __LED0_ON_L + 4 * channel, alist )
        return False
    except:
        print "Exception error"
        return True

def getPWMquad(address, channel, length):
    # Gets a single PWM channel values
    # Channels are 0-15 for individual settings and 61 returns [0,0,0,0]
    # Quad register List is [ON_lsb, ON_msb, OFF_lsb, OFF_msb]
    try:
        return busReadlist(address, __LED0_ON_L + 4 * channel, length )
    except:
        print "Exception error"
        return True

#*****************************************************
#Test code

def main():
    print "Starting Test on channels 0-7"

    PCA9685_reset()

    setPWMFreq(60)

    #Set neutral on all channels using register 61, (0xFA)
    setPWMquad(__PCA9685add, 61, __Neutral)
    while True:
        try:
            sleep (0.100)
            newquad= [0,0, randrange(300, 450, 4),randrange(0, 3, 1)]
            newchan= randrange(0, 8, 1)
            setPWMquad(__PCA9685add, newchan, newquad)
        except KeyboardInterrupt:
            print "Keyboard Interrupt"
            setPWMquad(__PCA9685add, 61, __Neutral)
            break

main()




    
