#!/usr/bin/python

import sys
from time import time, sleep
from subprocess import call, Popen, PIPE, STDOUT

piperef=[0,1,2]
rcam=0
rwheel=1
rbuttons=2
leftwheel=0
rightwheel=1
left=0
right=1
neutral=370
LEDflash=False
LEDon=False
basespeed=0         #add and subtract from neutral
steerspeed=0        #add to one wheel and subtract from other
steerdir=left       #0=left 1=right
nosteer=1           #0=steer 1=no steer
paused=True         #if True then drop camera and wheel actions

printon=False

def initpipe(pipenum, progstr, sudoflag):
    global piperef
    try:
        with open( progstr + '_stderr_' + str(pipenum) + '.txt', 'wb') as err:
            if sudoflag :
                piperef[pipenum]=Popen(['sudo', 'python', progstr],
                                   stdout=PIPE, stdin=PIPE, stderr=err)
            else:
                piperef[pipenum]=Popen(['python', progstr],
                                   stdout=PIPE, stdin=PIPE, stderr=err)
            x = piperef[pipenum].stdout.readline()
            if 'ok:' in x:
                print progstr + ' ready'
                return False
            else:
                if printon : print 'Wrong response from child ', progstr, x
                return True
    except IOError:
        if printon : print 'IOError on init....'
        return True

    
def pingpipe(pipenum, sendstr):
    global piperef
    try:
        piperef[pipenum].stdin.write(sendstr)
        piperef[pipenum].stdin.flush()
        return piperef[pipenum].stdout.readline()
    except IOError:
        if printon : print 'IOError on pipe send ', pipenum
        return True

def startpipes():
    if  initpipe(rbuttons, 'rbuttons.py', True):
        #an Error occured
        if printon : print 'Error creating pipe to ', rbuttons
        return True
    if  initpipe(rcam, 'rcam.py', True):
        #an Error occured
        if printon : print 'Error creating pipe to ', rcam
        return True
    if  initpipe(rwheel, 'rwheel.py', False):
        #an Error occured
        if printon : print 'Error creating pipe to ', rwheel
        return True
    return False

def flashLED():
    global LEDflash
    LEDflash = not LEDflash
    if LEDflash and not LEDon :
        LEDstr='LEDoff\n'
    else:
        LEDstr='LEDon\n'
    x = pingpipe(rbuttons, LEDstr )
    if x == True:
        if printon : print 'Error on pipe send ', rbuttons
    else:
        if printon : print 'Pipe=', rbuttons, x,

def sensebuttons():
    global paused
    x = pingpipe(rbuttons, 'rdswevents\n' )
    if x == True:
        print 'Error on pipe send ', rbuttons
    else:
        if 'ok:' in x and x[11] == ']':
            y=eval(x[0:12])
            if y[0] :           #GPIO(21) do shutdown OS
                z=Popen(['sudo', 'shutdown', '-h', 'now'])
                print 'Shutdown in process'
                return
            if y[1] :           #GPIO(20) toggle paused flag
                paused = not paused
        else:
            if printon : print 'Return string error '

def readcamera():
    global steerdir
    global steerspeed
    global nosteer
    global basespeed
    x = pingpipe(rcam, 'line0\n' )
    if x == True:
        print 'Error on pipe send ', camref
    else:
        if printon : print 'Pipe=', camref, x,
        if ', ok:' in x :
            y=eval(x[0:(len(x)-6)]) #convert truncated str to tuple
            z = int(y[5])
            if printon : print z
        if z >= 47 and z <= 52:       #deadzone
            #no steer action, raise basespeed
            if basespeed <= 45 : basespeed += 3
            if basespeed >= 45 : basespeed = 45
            steerspeed = 0
            nosteer=1
        #Stage 1
        elif (z >= 40 and z <= 47) or (z <= 60 and z >= 52 ):
            if z >= 52 :  steerdir = right   #Steer right
            if z <= 47 :  steerdir = left    #Steer left
            basespeed = 15          #Go slow       
            steerspeed = 6
        #Stage 2
        elif z <= 40 or z >= 60:
            if z >= 52 :  steerdir = right   #Steer right
            if z <= 47 :  steerdir = left    #Steer left
            basespeed = 10          #Go slower
            steerspeed = 9
           
        if printon : print basespeed, steerspeed, steerdir, nosteer,

    
def setwheel(wheelnum, speed):
    outstr = 'servowr,' + str(wheelnum) + ',' + str(speed)  + '\n'
    x = pingpipe(rwheel, outstr )
    if x == True:
        if printon : print 'Error on pipe send ', rwheel
    else:
        if printon : print 'Pipe=', rwheel, x,

def setLRspeed():
        #Set Left and Right wheel speed
    if steerdir :       #slew right
        lspeed = neutral - (basespeed - steerspeed*nosteer)            
        rspeed = neutral + (basespeed + steerspeed*nosteer)            
    else:               #slew left
        lspeed = neutral - (basespeed + steerspeed*nosteer)
        rspeed = neutral + (basespeed - steerspeed*nosteer)
    setwheel(leftwheel, lspeed)
    setwheel(rightwheel, rspeed)
    if printon : print 'LR speed ', lspeed, rspeed

def tidyexit():
    global piperef
    print '\nTidy exit...'
    x = pingpipe(rcam, 'close\n' )
    sleep(0.20)
    x = pingpipe(rbuttons, 'close\n' )
    sleep(0.20)
    x = pingpipe(rwheel, 'close\n' )
    sleep(0.20)
    sys.exit('Closing application')


def main():
    if startpipes():
        print 'Aborting....something did not start!'
        sys.exit('Closing application')
    else:
        print 'Press Ctl-C to exit application.'
    #Start scanning loop
    while True :
        try:
            start=time()
            #Button and LED commands
            sensebuttons()
            flashLED()
            if paused :
                print ' Paused....             \r',
                setwheel(leftwheel, neutral)
                setwheel(rightwheel, neutral)
                sleep(0.04)
            else:
                #Camera commands
                readcamera()
                #Wheel commands
                setLRspeed()
                #Calculate loop time
                stop=time()
                print ' Loop = ', str(round((1 / (stop-start)), 2)), '\r',
        except KeyboardInterrupt :
            break
    #Tidy up before we leave
    tidyexit()            
    

if __name__ ==  '__main__':

    main()
    
