#!/usr/bin/python

#Limited to Python2 because of the PIL

import io, sys, picamera
from PIL import Image

camwidth=100
camheight=20
camfps=60
cam=0
imarray=[(0,0,0,0,0,0)]*camheight
stream=0
threshhold=80
campreview=False

def flushout(mystring):
    sys.stdout.write(mystring)
    sys.stdout.flush()

#initialize the camera and set parameters for capture
def initcam(width,height,fps):
    global cam
    global stream
    cam = picamera.PiCamera()
    cam.resolution = (width,height)
    cam.framerate = fps
    if campreview : cam.start_preview()
    stream=io.BytesIO()


def closecam():
    global cam
    cam.close()
    sys.exit('Bye....')


def acquire():
    #acquire and process the lines in one frame
    global imarray
    global stream
    try:
        # Initialize the stream object and capture the stream
        stream.seek(0)
        cam.capture(stream, use_video_port=True, format='bmp' )
        stream.seek(0)
        # Create an image from the stream
        im=Image.open(stream)
        px_val=list(im.getdata())
        # Initialize our output array of processed results
        for x in range(camheight):
            imarray[x]=(0,0,0,0,0,(camwidth/2))
        # We don't need to process every line in the capture
        # You can either do every third as below or select just
        # the lines you want to process using 'range'
        #        for y in range(0, camheight, 3):
        
        # In this case we will only process line 0 and 18
        for y in 0, 18 :
            seqcount=0
            oldseqmax=0
            oldseqx=0
            for x in range(0, camwidth):
                R,G,B=px_val[y*camwidth + x ]
                if R < threshhold and G < threshhold and B < threshhold:
                    seqcount += 1
                    if seqcount > oldseqmax :
                        oldseqmax = seqcount
                        oldseqx = x
                    a,b,c,d,e,f = imarray[y]
                    a += (R+G+B)/3      #average color - sum
                    b += 1              #target pixel count
                    c = oldseqmax       #max count of adjacent pixels
                    d = oldseqx         #max x positon of adjacent
                    e += x              #Sensor channel sum
                    if x <= camwidth/2: #steering value - 50=neutral
                        f -= 1
                    else:
                        f += 1
                    imarray[y]=(a,b,c,d,e,f)
                else:
                    seqpxcount=0
        return False
    except IOError:
        return True

#Command definitions
def line0():
    x=acquire()    
    if x == True:      #an error
        flushout('Error 1 ')
    else:
        flushout (str(imarray[0]) + ', ' )

def line18():
    x=acquire()    
    if x == True:      #an error
        flushout('Error 18 ')
    else:
        flushout(str(imarray[18]) + ', ')
  
def line018():
    x=acquire()    
    if x == True:      #an error
        flushout('Error 018 ')
    else:
#        flushout(str(imarray))
        flushout(str(imarray[0]) + ', ' + str(imarray[18]) + ', ')

    
def main():
    initcam(camwidth,camheight,camfps)

    while True:
        flushout('ok:\n')
        global cmddata
        try:
            inputstr=raw_input()
            inputstr += ',0,0'                  #need at least one item
            cmddata = eval(inputstr,{'__builtins__':None},
                           {'line0':line0,
                            'line18':line18,
                            'line018':line018,
                            'close':closecam })
            cmddata[0]()                        #execute the command
        except (KeyboardInterrupt, EOFError, NameError, SyntaxError, IndexError, TypeError) as err:
            if 'KeyboardInterrupt' in str(type(err)): break
            if 'EOFError' in str(type(err)): break
            flushout(str(err) + '  ')


if __name__ ==  '__main__':
    
    main()
 
