#!/usr/bin/python

import sys
from time import sleep
from random import randrange
from smbus import SMBus

# RPi - PCA9685 Constants 
__i2cbus             = 0x01     #older Pi may use bus 0 = 0x00
__Gencall            = 0x00
__PCA9685add         = 0x40
__PCA9685rst         = 0x06     #GenCall(0x06) resets all PCA9685
__MODE1              = 0x00
__PRESCALE           = 0xFE
__LED0_ON_L          = 0x06     #first register in quad
__Neutral            = [ 0x00, 0x00, 0x72, 0x01 ]  #1.5mS=370=0x172
__Quadlen            = 0x04

#holder for i2c SMBus object reference
i2c = 0

def flushout(mystring):
    sys.stdout.write(mystring)
    sys.stdout.flush()


#****************************************************
#SMBus functions

def busWritebyte( address, reg):
    #Adresses a single special register  
    try:
        return i2c.write_byte(address, reg)
    except IOError, err:
        flushout( "Exception error a" )
        return True   

def busReadbytedata (address, reg ):
    #Reads a single bye from a register
    try:
        return i2c.read_byte_data(address, reg)
    except IOError, err :
        flushout( "Exception error b" )
        return True

def busWritebytedata (address, reg, data):
    #Writes a single byte to a register
    try:
        i2c.write_byte_data(address, reg, data)
        return False
    except IOError, err :
        flushout( "Exception error c" )
        return True

def busWritelist(address, reg, alist):      
    #Writes an array of bytes in a list (needs AI)
    try:
        i2c.write_i2c_block_data(address, reg, alist)
        return False
    except IOError, err :
        flushout( "Exception error d" )
        return True

def busReadlist(address, reg, listlen):
    #Read multiple bytes into an array (needs AI)
    try:
        return i2c.read_i2c_block_data(address, reg, listlen)
    except IOError, err :
        flushout( "Exception error e" )
        return True

#******************************************
#PCA9685 specific functions
    
def PCA9685_reset():
    try:
        busWritebyte(__Gencall, __PCA9685rst) 
        return False
    except:
        flushout( "Exception error 1" )
        return True

def setPWMFreq(freq):
    # Sets the PWM frequency for this board
    prescale = 25000000.0    #25MHz rc osc
    prescale /= 4096.0       #12-bit resolution
    prescale /= float(freq)
    prescale -= 1.0
    prescale = int(prescale + 0.5)
    # the PCA9685 must be asleep to set the prescale divider count
    try:
        mode1data = busReadbytedata(__PCA9685add, __MODE1)   #read current MODE1 reg
        if not (mode1data & 0x10) :
            mode1data = mode1data | 0x10                         # sleep bit not set
            busWritebytedata(__PCA9685add, __MODE1, mode1data)   #MODE1 write sleep
        busWritebytedata(__PCA9685add, __PRESCALE, prescale)  #write prescale count
        mode1data=mode1data & 0xEF | 0x20          #MODE1 set AI, clear SLEEP
        busWritebytedata(__PCA9685add, __MODE1, mode1data)      #MODE1 write
        sleep(0.001)                                            #wait time for osc
        return False
    except:
        flushout( "Exception error 2" )
        return True

def setPWMquad(address, channel, alist):
    # Sets a single PWM channel, count can be 0-4095
    # Channels are 0-15 for individual settings and 61 for ALL_LED_ON_L
    # Quad register List is [ON_lsb, ON_msb, OFF_lsb, OFF_msb]
    try:
        busWritelist(address, __LED0_ON_L + 4 * channel, alist )
        return False
    except:
        flushout( "Exception error 3" )
        return True

def getPWMquad(address, channel):
    # Gets a single PWM channel value
    # Channels are 0-15 for individual settings and 61 returns [0,0,0,0]
    # Quad register List is [ON_lsb, ON_msb, OFF_lsb, OFF_msb]
    try:
        return busReadlist(address, __LED0_ON_L + 4 * channel, __Quadlen )
    except:
        flushout( "Exception error 4" )
        return True

#*****************************************************
#Command definitions

def servonuetral():
    x = setPWMquad(__PCA9685add, 61, __Neutral)
    if x == True:      #an error
        flushout( "Exception error 5" )

def servowr():
    global cmddata
    newquad= [0,0, cmddata[2] & 255, cmddata[2] >> 8 & 255]
    x = setPWMquad(__PCA9685add, cmddata[1], newquad)
    if x == True:      #an error
        flushout("Exception error 6")

def servord():
    global cmddata
    x = getPWMquad(__PCA9685add, cmddata[1])
    if x == True:      #an error
        flushout("Exception error 7")
    else:
        flushout(str(x[2] + x[3] * 256) + ', ')

def closeandexit():
    setPWMquad(__PCA9685add, 61, __Neutral)
    sys.exit('Bye....')


def main():
    global i2c
    i2c = SMBus(__i2cbus)
    PCA9685_reset()
    setPWMFreq(60)
    #Set neutral on all channels 
    servonuetral()

    while True:
        flushout('ok:\n')
        global cmddata
        try:
            inputstr=raw_input()
            inputstr += ',0,0'                  #need at least one item
            cmddata = eval(inputstr,{'__builtins__':None},
                           {'servonuetral':servonuetral,
                            'servowr':servowr,
                            'servord':servord,
                            'close':closeandexit })
            cmddata[0]()                        #execute the command
        except (KeyboardInterrupt, NameError, SyntaxError, IndexError, TypeError) as err:
            if 'KeyboardInterrupt' in str(type(err)):
                setPWMquad(__PCA9685add, 61, __Neutral)
                break
            flushout(str(err) + '  ')


if __name__ ==  '__main__':
    
    main()



    
